<?php
namespace tmc\revisionmanager\src\Components;

/**
 * @author jakubkuranda@gmail.com
 * Date: 08.03.2018
 * Time: 12:49
 */

use tmc\revisionmanager\src\App;
use WP_Post;
use WP_Query;
use WP_User;

class Revisions {

    const CREATE_LINKED_POST_ACTION_NAME        = 'rm_tmc_create_linked_post';
    const DELETE_ALL_REVISIONS_ACTION_NAME      = 'rm_tmc_delete_all_revisions';
    const SEND_QUICK_EMAIL_TEST_ACTION_NAME     = 'rm_tmc_quickEmailTest';

    /** @var int|null */
    public $linkedId;

    /**
     * Revision handler constructor.
     * It registers some actions, etc.
     */
    public function __construct() {

        //  ----------------------------------------
        //  Actions
        //  ----------------------------------------

        add_action( 'admin_notices', 										    array( $this, '_a_displayPostsMergeNotice' ) );
        add_action( 'post_submitbox_misc_actions',                              array( $this, '_a_submitboxActionsInfo' ) );

        add_action( 'admin_action_' . $this::CREATE_LINKED_POST_ACTION_NAME,    array( $this, '_a_createLinkedPost' ) );

        add_action( 'pending_to_publish', 									    array( $this, '_a_publishClonedPost' ) );
        add_action( 'draft_to_publish', 									    array( $this, '_a_publishClonedPost' ) );

        add_action( 'save_post', 											    array( $this, '_a_sendNotificationAboutPendingPostClone' ) );

        add_action( 'wp_ajax_' . $this::SEND_QUICK_EMAIL_TEST_ACTION_NAME,      array( $this, '_a_ajaxQuickEmailTestCallback' ) );
        add_action( 'wp_ajax_' . $this::DELETE_ALL_REVISIONS_ACTION_NAME,       array( $this, '_a_ajaxDeleteAllRevisionsCallback' ) );

        //  ----------------------------------------
        //  Filters
        //  ----------------------------------------

        add_filter(	'post_row_actions',										    array( $this, '_f_addActionLinksToPostsTable' ), 10, 2 );
        add_filter(	'page_row_actions',										    array( $this, '_f_addActionLinksToPostsTable' ), 10, 2 );
        add_filter( 'display_post_states',                                      array( $this, '_f_modifyPostStates' ), 20, 2 );

    }

    //  ================================================================================
    //  ACTIONS
    //  ================================================================================

    /**
     * Called on admin_notices.
     * Displays notification, if it's possible to merge posts.
     *
     * @return void
     */
    public function _a_displayPostsMergeNotice() {

        /** @var WP_Post $post */
        global $post;

        $currentScreen = get_current_screen();

        if( $post && $currentScreen && $currentScreen->base == 'post' && $currentScreen->parent_base == 'edit' ){

            $postTypes = App::i()->settings->getChosenPostTypesSlugs();

            if( in_array( $post->post_type, $postTypes ) ){

                $linkedPostId       = get_post_meta( $post->ID, 'linked_post_id', true );
                $linkedPostStatus   = get_post_status( $linkedPostId );

                if( $linkedPostId && $linkedPostStatus && in_array( $post->post_status, array( 'pending', 'draft' ) ) ){

                    //  Finally display notice

                    printf( '<div class="notice notice-info is-dismissible"><p><i class="dashicons dashicons-external"></i> %1$s</p></div>',
                        __( 'Publishing this post will override its original.', 'rm_tmc' )
                    );

                }

            }

        }

    }

    /**
     * Called on custom admin_action hook.
     * Creates duplicate of post and redirects to edition page.
     *
     * @return void
     */
    public function _a_createLinkedPost() {

        $originalPostId = isset( $_GET['post'] ) ? intval( $_GET['post'] ) : null;
        $linkedPostId = null;

        $post = get_post( $originalPostId );

        if( $post ){    //  If such post exists

            $linkedPostId = App::i()->utilities->copyPost( $post->ID, null, array( 'post_status' => 'draft' ) );

            if( $linkedPostId && ! is_wp_error( $linkedPostId ) ){

                //  Link both posts together
                update_post_meta( $post->ID, 'linked_post_id', $linkedPostId );
                update_post_meta( $linkedPostId, 'linked_post_id', $post->ID );

            } else {

                wp_die( __( 'Something went wrong while creation of post duplicate.', 'rm_tmc' ) );

            }

        } else {

            wp_die( __( 'Wrong post ID', 'rm_tmc' ) );

        }

        wp_safe_redirect( admin_url( 'post.php?action=edit&post=' . $linkedPostId ) );
        exit;

    }

    //	==============================================
    //	ACCEPT REVISION
    //	==============================================

    /**
     * Called on save_post hook.
     *
     * @param int $postId
     *
     * @return void
     */
    function _a_mergeAcceptedPost( $postId ) {

        if( $postId === $this->linkedId ){

            remove_action( 'save_post', array( $this, '_a_mergeAcceptedPost' ) );

            $linkedPostId   = get_post_meta( $postId, 'linked_post_id', true );
            $linkedPost     = get_post( $linkedPostId );

            $args = array(
                'post_status'	=>  'publish',
                'post_author'	=>  $linkedPost->post_author,
                'post_name'		=>  $linkedPost->post_name
            );

            App::i()->utilities->copyPost( $postId, $linkedPostId, $args );
            wp_delete_post( $postId, true );

            wp_redirect( admin_url( 'edit.php?post_type=' . $linkedPost->post_type ) );
            exit;

        }

    }

    //	==============================================
    //	DETECT REVISION PUBLICATION
    //	==============================================

    /**
     * Called on post status change event ( from pending/draft to publish ).
     *
     * @param WP_Post $post
     *
     * @return void
     */
    function _a_publishClonedPost( $post ) {

        $postTypes = App::i()->settings->getChosenPostTypesSlugs();

        if( in_array( $post->post_type, $postTypes ) ){

            $linkedPostId       = get_post_meta( $post->ID, 'linked_post_id', true );
            $linkedPostStatus   = get_post_status( $linkedPostId );

            if( $linkedPostId && $linkedPostStatus ){

                $this->linkedId = $post->ID;   //  Set current post id as currently processed
                add_action( 'save_post', array( $this, '_a_mergeAcceptedPost' ) );

            }

        }

    }

    //	==============================================
    //	MAIL NOTIFICATION
    //	==============================================

    /**
     * Called on save_post hook.
     * Checks if this post is saved as pending.
     *
     * @param int $postId
     *
     * @return void
     */
    function _a_sendNotificationAboutPendingPostClone( $postId ) {

        $post       = get_post( $postId );
        $postTypes  = App::i()->settings->getChosenPostTypesSlugs();

        if(
            ! wp_is_post_revision( $postId )                //  Not a revision?
            && ! wp_is_post_autosave( $postId )             //  Not a autosave?
            && $post->post_status === 'pending'			    //  Is pending?
            && in_array( $post->post_type, $postTypes )     //  Is chosen by user?
        ){

            $targetUsers    = get_users( array( 'role' => App::i()->settings->getRoleForNotifications() ) );
            $excludedEmails = App::i()->settings->getExcludedEmailsFromNotifications();

            //  Prepare message ( replace extra codes with data )

            $subject = App::i()->settings->getNotificationSubject();
            $content = apply_filters( 'the_content', App::i()->settings->getNotificationContent() );

            $subject = App::i()->utilities->replaceCodes( $subject, $post );
            $content = App::i()->utilities->replaceCodes( $content, $post );

            //  Send to all targets

            foreach( $targetUsers as $targetUser ){ /** @var WP_User $targetUser */

                $targetEmail = $targetUser->user_email;

                if( ! in_array( $targetEmail, $excludedEmails ) ){

                    $html = sprintf( '<html><head></head><body>%1$s</body></head></html>', $content );

                    App::s()->messages->sendEmail( $subject, $html, $html );

                }

            }

            wp_reset_postdata();

        }

    }

    /**
     * Called on admin ajax hook rm_tmc_quickEmailTest.
     * Sends fake revision test mail.
     */
    public function _a_ajaxQuickEmailTestCallback() {

        //  ----------------------------------------
        //  Prepare data
        //  ----------------------------------------

        $emailTarget    = array_key_exists( 'emailTarget', $_POST ) ? sanitize_email( $_POST['emailTarget'] ) : '';
        $emailSubject   = array_key_exists( 'emailSubject', $_POST ) ? sanitize_text_field( $_POST['emailSubject'] ) : '';
        $emailContent   = array_key_exists( 'emailContent', $_POST ) ? apply_filters( 'the_content', wp_unslash( $_POST['emailContent'] ) ) : '';

        //  Modify with random post

        $randomPost = App::i()->utilities->getRandomPost();

        if( $randomPost ){
            $emailSubject = App::i()->utilities->replaceCodes( $emailSubject, $randomPost );
            $emailContent = App::i()->utilities->replaceCodes( $emailContent, $randomPost );
        }

        //  ----------------------------------------
        //  Send message
        //  ----------------------------------------

        $html = sprintf( '<html><head></head><body>%1$s</body></head></html>', $emailContent );

        $result = App::s()->messages->sendEmail( $emailSubject, $html, $emailTarget );

        wp_die( (int) $result );    //  Return 0/1

    }

    /**
     * Called on admin ajax hook.
     * Deletes all revisions.
     * Require 'manage_options' capability.
     */
    public function _a_ajaxDeleteAllRevisionsCallback() {

        if( current_user_can( App::i()->settings->getCapabilityForAcceptingChanges() ) ){

            $query = new WP_Query( array(
                'post_type'         =>  'revision',
                'post_status'       =>  'inherit',
                'nopaging'          =>  true,
                'fields'            =>  'ids'
            ) );

            $countDeleted = 0;

            foreach( $query->get_posts() as $postId ){  /** @var int $postId */

                $result = wp_delete_post_revision( $postId );

                if( $result && ! is_wp_error( $result ) ){
                    $countDeleted++;
                }

            }

            wp_die( sprintf( __( 'Removed <code>%1$s</code> revisions from database.', 'rm_tmc' ), $countDeleted ) );

        } else {

            wp_die( __( 'Sorry, but you are not allowed to do that.',   'rm_tmc' ) );

        }

    }

    /**
     * Adds info about publishing and setting post status to pending above submit button.
     *
     * @param WP_Post $post
     *
     * @return void
     */
    public function _a_submitboxActionsInfo( $post ) {

        $chosenPostTypes = App::i()->settings->getChosenPostTypesSlugs();

        if( in_array( $post->post_type, $chosenPostTypes ) && $post->post_status !== 'publish' ){

            $linkedPostId       = get_post_meta( $post->ID, 'linked_post_id', true );
            $linkedPostStatus   = get_post_status( $linkedPostId );

            if( $linkedPostId && $linkedPostStatus ){

                if( current_user_can( 'publish_posts' ) ){

                    printf( '<div class="misc-pub-section" style="color: #16a085;"><i class="dashicons dashicons-info"></i> %1$s</div>',
                        __( 'If you publish this post, it will override its original.', 'rm_tmc' )
                    );

                } else {

                    printf( '<div class="misc-pub-section" style="color: #16a085;"><i class="dashicons dashicons-email"></i> %1$s</div>',
                        __( 'If you change this post to pending, administrators will receive notification about it.', 'rm_tmc' )
                    );

                }

            }

        }

    }

    //  ================================================================================
    //  FILTERS
    //  ================================================================================

    /**
     * @param array $actions
     * @param WP_Post $post
     *
     * @return array
     */
    public function _f_addActionLinksToPostsTable( $actions, $post ) {

        $postTypes = App::i()->settings->getChosenPostTypesSlugs();

        if(
            in_array( get_post_type( $post ), $postTypes )	                    //  Current post type is chosen?
            && current_user_can( App::i()->settings->getCapabilityForCopyCreation() )   //  Current user can create post copies?
        ){

            $linkedPostId       = get_post_meta( $post->ID, 'linked_post_id', true );
            $linkedPostStatus   = get_post_status( $linkedPostId );

            //  ----------------------------------------
            //  Adding actions to row
            //  ----------------------------------------

            if(
                $linkedPostId
                && $post->post_status === 'publish'
                && in_array( $linkedPostStatus, array( 'pending', 'draft' ) )
            ){

                $actions['edit_linked_draft'] = sprintf(
                    '<a class="rm_tmc_row_action" href="post.php?post=%1$s&action=edit">
                                                        <i class="dashicons dashicons-welcome-edit-page"></i> %2$s
                                                    </a>',
                    $linkedPostId,
                    __( 'Edit connected revision draft', 'rm_tmc' )
                );

            } else if(
                $linkedPostId
                && in_array( $post->post_status, array( 'pending', 'draft' ) )
                && $linkedPostStatus === 'publish'
            ){

                $actions['show_linked_post'] =  sprintf(
                    '<a target="_blank" href="%1$s">
                                                        <i class="dashicons-before dashicons-visibility"></i> %2$s
                                                    </a>',
                    get_permalink( $linkedPostId ),
                    __( 'Show original post', 'rm_tmc' )
                );

            } else if( $post->post_status === 'publish' ){

                $actions['create_linked_draft'] =   sprintf(
                    '<a class="rm_tmc_row_action" href="admin.php?action=%1$s&post=%2$s">
                                                            <i class="dashicons dashicons-welcome-add-page"></i> %4$s
                                                        </a>',
                    $this::CREATE_LINKED_POST_ACTION_NAME,
                    $post->ID,
                    App::s()->getUrl( 'assets/img/tick.png' ),
                    __( 'Create revision draft', 'rm_tmc' )
                );

            }

        }

        return $actions;
    }

    /**
     * Modifies (replaces) default post states display on posts list.
     *
     * @param array $postStates
     * @param WP_Post $post
     *
     * @return array
     */
    public function _f_modifyPostStates( $postStates, $post ) {

        $chosenPostTypes = App::i()->settings->getChosenPostTypesSlugs();

        if( in_array( $post->post_type, $chosenPostTypes ) ){

            $linkedPostId       = get_post_meta( $post->ID, 'linked_post_id', true );
            $linkedPostStatus   = get_post_status( $linkedPostId );

            //  ----------------------------------------
            //  Modify state display
            //  ----------------------------------------

            if( $linkedPostId && $linkedPostStatus === 'publish' ){

                if( $post->post_status === 'pending' ){
                    $postStates['pending'] = __( 'Revision is waiting for acceptation.', 'rm_tmc' );
                }

                if( $post->post_status === 'draft' ){
                    $postStates['draft'] = __( 'Revision is waiting for acceptation.', 'rm_tmc' );
                }

            }

        }

        return $postStates;

    }

}